#!/usr/bin/perl 

package KShieldAuthLDAP ;
use Kernel::System::Auth::LDAP ;
@ISA = qw (Kernel::System::Auth::LDAP) ;

use Net::LDAP ;
use KShieldClient ;

sub new {
	my($class) = shift;
    my(%params) = @_;
    my($Self) = Kernel::System::Auth::LDAP->new(@_);
	

	my $ConfigObject = $Kernel::OM->Get('Kernel::Config');

  $Self->{serverURL} = $ConfigObject->Get( 'AuthModule::KShield::ServerURL' . $Param{Count} );
  $Self->{APIKey} = $ConfigObject->Get( 'AuthModule::KShield::APIKey' . $Param{Count} );
  $Self->{kshieldUserAttr} = $ConfigObject->Get( 'AuthModule::KShield::UserAttr' . $Param{Count} );
	
	 return (bless($Self, $class));
}

sub ObtainKShieldUID {
	my($Self) = @_ ;
	
	$wst = new KShieldClient($Self->{serverURL}, $Self->{APIKey}, $Self->{kshieldUserAttr}) ;
	
	my $RemoteAddr = $ENV{REMOTE_ADDR} ;
	
	return $wst->getByIP($RemoteAddr) ;
}

sub Auth {
    my ( $Self, %Param ) = @_;

	my $uid = $Self->ObtainKShieldUID ;
	my $fdn = $uid -> {fdn} ;
	
	if (!$Param{Pw} && $fdn && $Self->CheckGroupMembership($fdn)) {
		return $uid -> {screenName} ;
	}
	
	return $Self->SUPER::Auth(%Param) ;
}

sub GetOption {
	my ($Self, %Param) = @_ ;
	
	my $uid = $Self->ObtainKShieldUID ;
	
	
	my $fdn =  $uid -> {fdn} ;
	my $screenName = $uid -> {screenName} ;
	
	if ($fdn && $Self->CheckGroupMembership($fdn, $screenName)) {
		my %Option = ( PreAuth => 1, );
		return $Option{ $Param{What} };
	}
	
	print STDERR "fdn: $fdn" ;
	
	return $Self->SUPER::GetOption(%Param) ;
}

sub CheckGroupMembership {
	my ($Self, $UserDN, $User) = @_ ;

    # check if user need to be in a group!
    if ( $Self->{AccessAttr} && $Self->{GroupDN} ) {
		my $LDAP = $Self->ConnectToLDAP ;
		# DN quote
		if (!$LDAP) 
		{
			return ;
		}
		
		my $UserDNQuote = $UserDN;
		$UserDNQuote =~ s/\\/\\\\/g;
		$UserDNQuote =~ s/\(/\\(/g;
		$UserDNQuote =~ s/\)/\\)/g;
		
		my $UserQuote = $User;
		$UserQuote =~ s/\\/\\\\/g;
		$UserQuote =~ s/\(/\\(/g;
		$UserQuote =~ s/\)/\\)/g;
		

        # just in case for debug
        if ( $Self->{Debug} > 0 ) {
            $Self->{LogObject}->Log(
                Priority => 'notice',
                Message  => 'check for groupdn!',
            );
        }

        # search if we're allowed to
		my $Filter2 = '';
        if ( $Self->{UserAttr} eq 'DN' ) {
            $Filter2 = "($Self->{AccessAttr}=$UserDNQuote)";
        }
        else {
            $Filter2 = "($Self->{AccessAttr}=$UserQuote)";
        }
		            
        my $Result2 = $LDAP->search(
            base   => $Self->{GroupDN},
            filter => $Filter2,
            attrs  => ['1.1'],
        );
        if ( $Result2->code ) {
            $Self->{LogObject}->Log(
                Priority => 'error',
                Message  => "Search failed! base='$Self->{GroupDN}', filter='$Filter2', "
                    . $Result->error,
            );

            # take down session
            $LDAP->unbind;
            $LDAP->disconnect;
            return;
        }

        # extract it
        my $GroupDN = '';
        for my $Entry ( $Result2->all_entries ) {
            $GroupDN = $Entry->dn();
        }

        # log if there is no LDAP entry
        if ( !$GroupDN ) {

            # failed login note
            $Self->{LogObject}->Log(
                Priority => 'notice',
                Message  => "User: $Param{User} authentication failed, no LDAP group entry found"
                    . "GroupDN='$Self->{GroupDN}', Filter='$Filter2'! (REMOTE_ADDR: $RemoteAddr).",
            );

            # take down session
            $LDAP->unbind;
            $LDAP->disconnect;
            return;
        }
		
      $LDAP->unbind;
      $LDAP->disconnect;
    }
	
	  
	return 1 ;
}

sub ConnectToLDAP {
	my($Self) = @_ ;
	my $LDAP = Net::LDAP->new( $Self->{Host}, %{ $Self->{Params} } );
    if ( !$LDAP ) {
        if ( $Self->{Die} ) {
            die "Can't connect to $Self->{Host}: $@";
        }
        else {
            $Self->{LogObject}->Log(
                Priority => 'error',
                Message  => "Can't connect to $Self->{Host}: $@",
            );
            return;
        }
    }
    my $Result = '';
    if ( $Self->{SearchUserDN} && $Self->{SearchUserPw} ) {
        $Result = $LDAP->bind( dn => $Self->{SearchUserDN}, password => $Self->{SearchUserPw} );
    }
    else {
        $Result = $LDAP->bind();
    }
	
	if ( $Result->code ) {
        $Self->{LogObject}->Log(
            Priority => 'error',
            Message  => 'First bind failed! ' . $Result->error(),
        );
        $LDAP->disconnect;
        return;
    }
	
	
	return $LDAP;
}

1;